@@ -220,6 +220,7 @@ client_start (NMDhcpManager *self,
               const char *hostname,
               gboolean hostname_use_fqdn,
               NMDhcpHostnameFlags hostname_flags,
+              const char *mud_url,
               gboolean info_only,
               NMSettingIP6ConfigPrivacy privacy,
               const char *last_ip4_address,
@@ -239,21 +240,27 @@ client_start (NMDhcpManager *self,
 	g_return_val_if_fail (!dhcp_client_id || g_bytes_get_size (dhcp_client_id) >= 2, NULL);
 	g_return_val_if_fail (!error || !*error, NULL);
 
-	if (!hwaddr || !bcast_hwaddr) {
-		nm_utils_error_set (error,
-		                    NM_UTILS_ERROR_UNKNOWN,
-		                    "missing %s address",
-		                    hwaddr ? "broadcast" : "MAC");
-		return NULL;
-	}
+	if (addr_family == AF_INET) {
+		if (!hwaddr || !bcast_hwaddr) {
+			nm_utils_error_set (error,
+			                    NM_UTILS_ERROR_UNKNOWN,
+			                    "missing %s address",
+			                    hwaddr ? "broadcast" : "MAC");
+			return NULL;
+		}
 
-	hwaddr_len = g_bytes_get_size (hwaddr);
-	if (   hwaddr_len == 0
-	    || hwaddr_len > NM_UTILS_HWADDR_LEN_MAX) {
-		nm_utils_error_set (error,
-		                    NM_UTILS_ERROR_UNKNOWN,
-		                    "invalid MAC address");
-		g_return_val_if_reached (NULL) ;
+		hwaddr_len = g_bytes_get_size (hwaddr);
+		if (   hwaddr_len == 0
+		    || hwaddr_len > NM_UTILS_HWADDR_LEN_MAX) {
+			nm_utils_error_set (error,
+			                    NM_UTILS_ERROR_UNKNOWN,
+			                    "invalid MAC address");
+			g_return_val_if_reached (NULL) ;
+		}
+		nm_assert (g_bytes_get_size (hwaddr) == g_bytes_get_size (bcast_hwaddr));
+	} else {
+		hwaddr = NULL;
+		bcast_hwaddr = NULL;
 	}
 
 	if (hostname) {
@@ -267,8 +274,6 @@ client_start (NMDhcpManager *self,
 		}
 	}
 
-	nm_assert (g_bytes_get_size (hwaddr) == g_bytes_get_size (bcast_hwaddr));
-
 	priv = NM_DHCP_MANAGER_GET_PRIVATE (self);
 
 	/* Kill any old client instance */
@@ -303,6 +308,7 @@ client_start (NMDhcpManager *self,
 	                       NM_DHCP_CLIENT_IAID, (guint) iaid,
 	                       NM_DHCP_CLIENT_IAID_EXPLICIT, iaid_explicit,
 	                       NM_DHCP_CLIENT_HOSTNAME, hostname,
+	                       NM_DHCP_CLIENT_MUD_URL, mud_url,
 	                       NM_DHCP_CLIENT_ROUTE_TABLE, (guint) route_table,
 	                       NM_DHCP_CLIENT_ROUTE_METRIC, (guint) route_metric,
 	                       NM_DHCP_CLIENT_TIMEOUT, (guint) timeout,
@@ -382,6 +388,7 @@ nm_dhcp_manager_start_ip4 (NMDhcpManager *self,
                            const char *dhcp_hostname,
                            const char *dhcp_fqdn,
                            NMDhcpHostnameFlags hostname_flags,
+                           const char *mud_url,
                            GBytes *dhcp_client_id,
                            guint32 timeout,
                            const char *dhcp_anycast_addr,
@@ -440,6 +447,7 @@ nm_dhcp_manager_start_ip4 (NMDhcpManager *self,
 	                     hostname,
 	                     use_fqdn,
 	                     hostname_flags,
+	                     mud_url,
 	                     FALSE,
 	                     0,
 	                     last_ip_address,
@@ -453,8 +461,6 @@ nm_dhcp_manager_start_ip6 (NMDhcpManager *self,
                            NMDedupMultiIndex *multi_idx,
                            const char *iface,
                            int ifindex,
-                           GBytes *hwaddr,
-                           GBytes *bcast_hwaddr,
                            const struct in6_addr *ll_addr,
                            const char *uuid,
                            guint32 route_table,
@@ -462,6 +468,7 @@ nm_dhcp_manager_start_ip6 (NMDhcpManager *self,
                            gboolean send_hostname,
                            const char *dhcp_hostname,
                            NMDhcpHostnameFlags hostname_flags,
+                           const char *mud_url,
                            GBytes *duid,
                            gboolean enforce_duid,
                            guint32 iaid,
@@ -488,8 +495,8 @@ nm_dhcp_manager_start_ip6 (NMDhcpManager *self,
 	                     multi_idx,
 	                     iface,
 	                     ifindex,
-	                     hwaddr,
-	                     bcast_hwaddr,
+	                     NULL,
+	                     NULL,
 	                     uuid,
 	                     route_table,
 	                     route_metric,
@@ -503,6 +510,7 @@ nm_dhcp_manager_start_ip6 (NMDhcpManager *self,
 	                     hostname,
 	                     TRUE,
 	                     hostname_flags,
+	                     mud_url,
 	                     info_only,
 	                     privacy,
 	                     NULL,
@@ -515,7 +523,7 @@ nm_dhcp_manager_set_default_hostname (NMDhcpManager *manager, const char *hostna
 {
 	NMDhcpManagerPrivate *priv = NM_DHCP_MANAGER_GET_PRIVATE (manager);
 
-	g_clear_pointer (&priv->default_hostname, g_free);
+	nm_clear_g_free (&priv->default_hostname);
 
 	/* Never send 'localhost'-type names to the DHCP server */
 	if (!nm_utils_is_specific_hostname (hostname))
