@@ -9,6 +9,8 @@
 #include "nm-connection.h"
 #include "nm-utils.h"
 
+#define NMI_WAIT_DEVICE_TIMEOUT_MS 60000
+
 static inline gboolean
 guess_ip_address_family (const char *str)
 {
@@ -28,6 +30,6 @@ gboolean nmi_ibft_update_connection_from_nic (NMConnection *connection, GHashTab
 
 NMConnection *nmi_dt_reader_parse (const char *sysfs_dir);
 
-GHashTable *nmi_cmdline_reader_parse (const char *sysfs_dir, const char *const*argv);
+GHashTable *nmi_cmdline_reader_parse (const char *sysfs_dir, const char *const*argv, char **hostname);
 
 #endif  /* __NM_INITRD_GENERATOR_H__ */
