@@ -1,48 +1,30 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
-/* NetworkManager -- Network link manager
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
- * Copyright (C) 2012 - 2017 Red Hat, Inc.
+// SPDX-License-Identifier: GPL-2.0+
+/*
+ * Copyright (C) 2012 - 2018 Red Hat, Inc.
  */
 
 #ifndef __NETWORKMANAGER_TYPES_H__
 #define __NETWORKMANAGER_TYPES_H__
 
-#ifdef __NM_UTILS_PRIVATE_H__
-#error "nm-utils-private.h" must not be used outside of libnm-core/. Do you want "nm-core-internal.h"?
+#if !((NETWORKMANAGER_COMPILATION) & NM_NETWORKMANAGER_COMPILATION_WITH_DAEMON)
+#error Cannot use this header.
 #endif
 
 #define _NM_SD_MAX_CLIENT_ID_LEN (sizeof (guint32) + 128)
 
 /* core */
-typedef struct _NMExportedObject     NMExportedObject;
+typedef struct _NMDBusObject         NMDBusObject;
 typedef struct _NMActiveConnection   NMActiveConnection;
 typedef struct _NMAuditManager       NMAuditManager;
 typedef struct _NMVpnConnection      NMVpnConnection;
 typedef struct _NMActRequest         NMActRequest;
 typedef struct _NMAuthSubject        NMAuthSubject;
-typedef struct _NMBusManager         NMBusManager;
+typedef struct _NMDBusManager        NMDBusManager;
 typedef struct _NMConfig             NMConfig;
 typedef struct _NMConfigData         NMConfigData;
-typedef struct _NMArpingManager      NMArpingManager;
-typedef struct _NMConnectionProvider NMConnectionProvider;
 typedef struct _NMConnectivity       NMConnectivity;
 typedef struct _NMDevice             NMDevice;
-typedef struct _NMDhcp4Config        NMDhcp4Config;
-typedef struct _NMDhcp6Config        NMDhcp6Config;
+typedef struct _NMDhcpConfig         NMDhcpConfig;
 typedef struct _NMProxyConfig        NMProxyConfig;
 typedef struct _NMIPConfig           NMIPConfig;
 typedef struct _NMIP4Config          NMIP4Config;
@@ -53,12 +35,21 @@ typedef struct _NMPolicy             NMPolicy;
 typedef struct _NMRfkillManager      NMRfkillManager;
 typedef struct _NMPacrunnerManager   NMPacrunnerManager;
 typedef struct _NMSessionMonitor     NMSessionMonitor;
+typedef struct _NMKeepAlive          NMKeepAlive;
 typedef struct _NMSleepMonitor       NMSleepMonitor;
 typedef struct _NMLldpListener       NMLldpListener;
 typedef struct _NMConfigDeviceStateData NMConfigDeviceStateData;
 
+typedef void (*NMManagerDeviceAuthRequestFunc) (NMDevice *device,
+                                                GDBusMethodInvocation *context,
+                                                NMAuthSubject *subject,
+                                                GError *error,
+                                                gpointer user_data);
+
 struct _NMDedupMultiIndex;
 
+typedef struct _NMRefString NMRefString;
+
 /*****************************************************************************/
 
 typedef enum {
@@ -75,6 +66,15 @@ typedef enum {
 	NM_ACTIVATION_TYPE_EXTERNAL = 2,
 } NMActivationType;
 
+typedef enum {
+	NM_ACTIVATION_REASON_UNSET,
+	NM_ACTIVATION_REASON_EXTERNAL,
+	NM_ACTIVATION_REASON_ASSUME,
+	NM_ACTIVATION_REASON_AUTOCONNECT,
+	NM_ACTIVATION_REASON_AUTOCONNECT_SLAVES,
+	NM_ACTIVATION_REASON_USER_REQUEST,
+} NMActivationReason;
+
 typedef enum {
 	/* In priority order; higher number == higher priority */
 
@@ -95,6 +95,7 @@ typedef enum {
 	NM_IP_CONFIG_SOURCE_KERNEL,
 	NM_IP_CONFIG_SOURCE_SHARED,
 	NM_IP_CONFIG_SOURCE_IP4LL,
+	NM_IP_CONFIG_SOURCE_IP6LL,
 	NM_IP_CONFIG_SOURCE_PPP,
 	NM_IP_CONFIG_SOURCE_WWAN,
 	NM_IP_CONFIG_SOURCE_VPN,
@@ -110,45 +111,51 @@ NM_IS_IP_CONFIG_SOURCE_RTPROT (NMIPConfigSource source)
 }
 
 /* platform */
-typedef struct _NMPlatform           NMPlatform;
-typedef struct _NMPlatformObject     NMPlatformObject;
-typedef struct _NMPlatformIP4Address NMPlatformIP4Address;
-typedef struct _NMPlatformIP4Route   NMPlatformIP4Route;
-typedef struct _NMPlatformIP6Address NMPlatformIP6Address;
-typedef struct _NMPlatformIP6Route   NMPlatformIP6Route;
-typedef struct _NMPlatformLink       NMPlatformLink;
-typedef struct _NMPNetns             NMPNetns;
-typedef struct _NMPObject            NMPObject;
+typedef struct _NMPlatform               NMPlatform;
+typedef struct _NMPlatformObject         NMPlatformObject;
+typedef struct _NMPlatformObjWithIfindex NMPlatformObjWithIfindex;
+typedef struct _NMPlatformIP4Address     NMPlatformIP4Address;
+typedef struct _NMPlatformIP4Route       NMPlatformIP4Route;
+typedef struct _NMPlatformIP6Address     NMPlatformIP6Address;
+typedef struct _NMPlatformIP6Route       NMPlatformIP6Route;
+typedef struct _NMPlatformLink           NMPlatformLink;
+typedef struct _NMPNetns                 NMPNetns;
+typedef struct _NMPObject                NMPObject;
 
 typedef enum {
-	/* Please don't interpret type numbers outside nm-platform and use functions
-	 * like nm_platform_link_is_software() and nm_platform_supports_slaves().
-	 *
-	 * type & 0x10000 -> Software device type
-	 * type & 0x20000 -> Type supports slaves
-	 */
 
 	/* No type, used as error value */
 	NM_LINK_TYPE_NONE,
 
-	/* Unknown type  */
 	NM_LINK_TYPE_UNKNOWN,
 
+	NM_LINK_TYPE_ANY,
+
+#define _NM_LINK_TYPE_REAL_FIRST NM_LINK_TYPE_ETHERNET
+
 	/* Hardware types */
+#define _NM_LINK_TYPE_HW_FIRST NM_LINK_TYPE_ETHERNET
 	NM_LINK_TYPE_ETHERNET,
 	NM_LINK_TYPE_INFINIBAND,
 	NM_LINK_TYPE_OLPC_MESH,
 	NM_LINK_TYPE_WIFI,
 	NM_LINK_TYPE_WWAN_NET,   /* WWAN kernel netdevice */
 	NM_LINK_TYPE_WIMAX,
+	NM_LINK_TYPE_WPAN,
+	NM_LINK_TYPE_6LOWPAN,
+	NM_LINK_TYPE_WIFI_P2P,
+#define _NM_LINK_TYPE_HW_LAST  NM_LINK_TYPE_WIFI_P2P
 
 	/* Software types */
-	NM_LINK_TYPE_BNEP = 0x10000,   /* Bluetooth Ethernet emulation */
+#define _NM_LINK_TYPE_SW_FIRST NM_LINK_TYPE_BNEP
+	NM_LINK_TYPE_BNEP,   /* Bluetooth Ethernet emulation */
 	NM_LINK_TYPE_DUMMY,
 	NM_LINK_TYPE_GRE,
 	NM_LINK_TYPE_GRETAP,
 	NM_LINK_TYPE_IFB,
 	NM_LINK_TYPE_IP6TNL,
+	NM_LINK_TYPE_IP6GRE,
+	NM_LINK_TYPE_IP6GRETAP,
 	NM_LINK_TYPE_IPIP,
 	NM_LINK_TYPE_LOOPBACK,
 	NM_LINK_TYPE_MACSEC,
@@ -157,20 +164,43 @@ typedef enum {
 	NM_LINK_TYPE_OPENVSWITCH,
 	NM_LINK_TYPE_PPP,
 	NM_LINK_TYPE_SIT,
-	NM_LINK_TYPE_TAP,
 	NM_LINK_TYPE_TUN,
 	NM_LINK_TYPE_VETH,
 	NM_LINK_TYPE_VLAN,
+	NM_LINK_TYPE_VRF,
 	NM_LINK_TYPE_VXLAN,
+	NM_LINK_TYPE_WIREGUARD,
+#define _NM_LINK_TYPE_SW_LAST  NM_LINK_TYPE_WIREGUARD
 
 	/* Software types with slaves */
-	NM_LINK_TYPE_BRIDGE = 0x10000 | 0x20000,
+#define _NM_LINK_TYPE_SW_MASTER_FIRST NM_LINK_TYPE_BRIDGE
+	NM_LINK_TYPE_BRIDGE,
 	NM_LINK_TYPE_BOND,
 	NM_LINK_TYPE_TEAM,
+#define _NM_LINK_TYPE_SW_MASTER_LAST NM_LINK_TYPE_TEAM
+
+#define _NM_LINK_TYPE_REAL_LAST NM_LINK_TYPE_TEAM
+
+#define _NM_LINK_TYPE_REAL_NUM ((int) (_NM_LINK_TYPE_REAL_LAST - _NM_LINK_TYPE_REAL_FIRST + 1))
 
-	NM_LINK_TYPE_ANY = G_MAXUINT32,
 } NMLinkType;
 
+static inline gboolean
+nm_link_type_is_software (NMLinkType link_type)
+{
+	G_STATIC_ASSERT (_NM_LINK_TYPE_SW_LAST + 1 == _NM_LINK_TYPE_SW_MASTER_FIRST);
+
+	return    link_type >= _NM_LINK_TYPE_SW_FIRST
+	       && link_type <= _NM_LINK_TYPE_SW_MASTER_LAST;
+}
+
+static inline gboolean
+nm_link_type_supports_slaves (NMLinkType link_type)
+{
+	return    link_type >= _NM_LINK_TYPE_SW_MASTER_FIRST
+	       && link_type <= _NM_LINK_TYPE_SW_MASTER_LAST;
+}
+
 typedef enum {
 	NMP_OBJECT_TYPE_UNKNOWN,
 	NMP_OBJECT_TYPE_LINK,
@@ -178,34 +208,51 @@ typedef enum {
 	NMP_OBJECT_TYPE_IP6_ADDRESS,
 	NMP_OBJECT_TYPE_IP4_ROUTE,
 	NMP_OBJECT_TYPE_IP6_ROUTE,
+	NMP_OBJECT_TYPE_ROUTING_RULE,
 
 	NMP_OBJECT_TYPE_QDISC,
 
 	NMP_OBJECT_TYPE_TFILTER,
 
 	NMP_OBJECT_TYPE_LNK_GRE,
+	NMP_OBJECT_TYPE_LNK_GRETAP,
 	NMP_OBJECT_TYPE_LNK_INFINIBAND,
 	NMP_OBJECT_TYPE_LNK_IP6TNL,
+	NMP_OBJECT_TYPE_LNK_IP6GRE,
+	NMP_OBJECT_TYPE_LNK_IP6GRETAP,
 	NMP_OBJECT_TYPE_LNK_IPIP,
 	NMP_OBJECT_TYPE_LNK_MACSEC,
 	NMP_OBJECT_TYPE_LNK_MACVLAN,
 	NMP_OBJECT_TYPE_LNK_MACVTAP,
 	NMP_OBJECT_TYPE_LNK_SIT,
+	NMP_OBJECT_TYPE_LNK_TUN,
 	NMP_OBJECT_TYPE_LNK_VLAN,
+	NMP_OBJECT_TYPE_LNK_VRF,
 	NMP_OBJECT_TYPE_LNK_VXLAN,
+	NMP_OBJECT_TYPE_LNK_WIREGUARD,
 
 	__NMP_OBJECT_TYPE_LAST,
 	NMP_OBJECT_TYPE_MAX = __NMP_OBJECT_TYPE_LAST - 1,
 } NMPObjectType;
 
+/**
+ * NMIPConfigMergeFlags:
+ * @NM_IP_CONFIG_MERGE_DEFAULT: no flags set
+ * @NM_IP_CONFIG_MERGE_NO_ROUTES: don't merge routes
+ * @NM_IP_CONFIG_MERGE_NO_DEFAULT_ROUTES: don't merge default routes.
+ *   Note that if the source IP config has NM_IP_CONFIG_FLAGS_IGNORE_MERGE_NO_DEFAULT_ROUTES
+ *   set, this flag gets ignored during merge.
+ * @NM_IP_CONFIG_MERGE_NO_DNS: don't merge DNS information
+ * @NM_IP_CONFIG_MERGE_EXTERNAL: mark new addresses as external
+ */
 typedef enum {
 	NM_IP_CONFIG_MERGE_DEFAULT                  = 0,
 	NM_IP_CONFIG_MERGE_NO_ROUTES                = (1LL << 0),
 	NM_IP_CONFIG_MERGE_NO_DEFAULT_ROUTES        = (1LL << 1),
 	NM_IP_CONFIG_MERGE_NO_DNS                   = (1LL << 2),
+	NM_IP_CONFIG_MERGE_EXTERNAL                 = (1LL << 3),
 } NMIPConfigMergeFlags;
 
-
 /**
  * NMIPRouteTableSyncMode:
  * @NM_IP_ROUTE_TABLE_SYNC_MODE_MAIN: only the main table is synced. For all
@@ -221,7 +268,6 @@ typedef enum {
 	NM_IP_ROUTE_TABLE_SYNC_MODE_ALL         = 3,
 } NMIPRouteTableSyncMode;
 
-
 /* settings */
 typedef struct _NMAgentManager       NMAgentManager;
 typedef struct _NMSecretAgent        NMSecretAgent;
@@ -231,4 +277,6 @@ typedef struct _NMSettingsConnection NMSettingsConnection;
 /* utils */
 typedef struct _NMUtilsIPv6IfaceId   NMUtilsIPv6IfaceId;
 
+#define NM_SETTING_CONNECTION_MDNS_UNKNOWN ((NMSettingConnectionMdns) -42)
+
 #endif  /* NM_TYPES_H */
